#include "IRLibAll.h" // Infrarot-Bibliothek

// Variablen der Motoren
int ENABLE = 4;
// Linker Getriebemotor
int IN1 = 9;
int IN2 = 10;
// Rechter Getriebemotor
int IN3 = 5;
int IN4 = 6;

// Geschwindigkeitsbestimmung
int pwm = 0

// Ultraschall
int trigger = 11;
int echo = 12;
long dauer = 0; // Zeitspeicherung (Schallwelle bis zur Reflektion und zurück.)
long entfernung = 0; // Speicherung der berechneten Entfernung

// Infrarot
IRrecvPCI myReceiver(3);
IRdecode myDecoder;

void setup() {
  Serial.begin(9600);
  // Ansteuerung Motoren
  pinMode(IN1, OUTPUT);
  pinMode(IN2, OUTPUT);
  pinMode(IN3, OUTPUT);
  pinMode(IN4, OUTPUT);
  pinMode(ENABLE, OUTPUT);

  // Ansteuerung Ultraschallmodul
  pinMode(trigger, OUTPUT); // Ausgang
  pinMode(echo, INPUT); // Eingang

  // Ansteuerung Infrarotsensor
  delay(2000); while (!Serial);
  myReceiver.enableIRIn(); // Start Receiver
  pinMode(3, OUTPUT);
}

// Motorbewegungen
void moveforward(int pwm) { // Vorwärtsbewegung
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  analogWrite(IN2, pwm);
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
  analogWrite(IN4, pwm);
}

void moveforwardright(int pwm) { // Rechtskurve vorwärts
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  analogWrite(IN2, 0);
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
  analogWrite(IN4, pwm);
}

void moveforwardleft(int pwm) { // Linkskurve vorwärts
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  analogWrite(IN2, pwm);
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
  analogWrite(IN4, 0);
}

void movebackward(int pwm) { // Rückwärtsbewegung
  digitalWrite(IN1, LOW);
  digitalWrite(IN2, HIGH);
  analogWrite(IN2, pwm);
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, HIGH);
  analogWrite(IN4, pwm);
}

void stopmove(int pwm) { // Anhalten
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  analogWrite(IN2, pwm);
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
  analogWrite(IN4, pwm);
}

void loop() {
  // Allgemeine Vorwärtsbewegung der Motoren
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  analogWrite(IN2, 50);
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
  analogWrite(IN4, 50);

  //Ultraschall
  digitalWrite(trigger, LOW); // Spannung des Trigger-Pin wird genommen, damit später rauschfreies Signal besteht.
  delay(5); // Dauer: 5 Millisekunden
  digitalWrite(trigger, HIGH); // Ultraschallwelle wird freigegeben...
  delay(10); // Dauer: 10 Millisekunden
  digitalWrite(trigger, LOW); // Ende Ultraschallwelle
  dauer = pulseIn(echo, HIGH); // Befehl „pulseIn“ zählt Mikrokontroller die Zeit(Mikrosekunden), bis Schall zurückkehrt.
  entfernung = (dauer / 2) * 0.03432; //Entfernungsrechnung(cm) = Zeit : 2 (Weil nur eine Strecke berechnet, nicht hin- und zurück)* Schallgeschwindigkeit cm/ms = cm
  
  // Auslesung des Infrarot-Lichtwertes
  if (entfernung >= 500 || entfernung <= 0)
  {
    Serial.println("Kein Messwert vorhanden");
  }
  else
  {
    Serial.print(entfernung);
    Serial.println(" cm");
  }
  delay(500);

  if (entfernung <= 60) { // Wenn Abstand geringer als 60 = anhalten
    stopmove(0);
  }
  if (entfernung >= 60) { // Wenn Abstand größer als 60 = fahren
    moveforward(50);
  }

  //Infrarot
  if (myReceiver.getResults()) {
    myDecoder.decode(); // Lichtwerte werden decodiert
    myDecoder.dumpResults(true); // Ausgabe Lichtwerte 
    myReceiver.enableIRIn(); // Receiver erneut gestartet
    Serial.println("detected");
  }
}


